@@ -1,4 +1,7 @@
 use crate::lockers::{StdinLocker, StdoutLocker};
+use async_std::fs::File;
+use async_std::io::{self, IoSlice, IoSliceMut, Read, Seek, Write};
+use async_std::net::TcpStream;
 #[cfg(unix)]
 use async_std::os::unix::{
     io::{AsRawFd, RawFd},
@@ -6,32 +9,27 @@ use async_std::os::unix::{
 };
 #[cfg(target_os = "wasi")]
 use async_std::os::wasi::io::{AsRawFd, RawFd};
-use async_std::{
-    fs::File,
-    io::{self, IoSlice, IoSliceMut, Read, Seek, Write},
-    net::TcpStream,
-};
 #[cfg(feature = "char-device")]
 use char_device::AsyncStdCharDevice;
 use duplex::Duplex;
-use std::{
-    fmt::{self, Debug},
-    pin::Pin,
-    task::{Context, Poll},
-};
+use io_lifetimes::{FromFilelike, FromSocketlike, IntoFilelike, IntoSocketlike};
+use std::fmt::{self, Debug};
+use std::pin::Pin;
+use std::task::{Context, Poll};
 use system_interface::io::ReadReady;
-#[cfg(not(windows))]
-use unsafe_io::os::posish::AsRawReadWriteFd;
 #[cfg(windows)]
 use unsafe_io::os::windows::{
-    AsRawHandleOrSocket, AsRawReadWriteHandleOrSocket, RawHandleOrSocket,
-};
-use unsafe_io::{
-    AsUnsafeHandle, AsUnsafeReadWriteHandle, FromUnsafeFile, FromUnsafeSocket, IntoUnsafeFile,
-    IntoUnsafeSocket, OwnsRaw,
+    AsHandleOrSocket, AsRawHandleOrSocket, AsRawReadWriteHandleOrSocket, AsReadWriteHandleOrSocket,
+    BorrowedHandleOrSocket, RawHandleOrSocket,
 };
+use unsafe_io::{AsRawGrip, AsRawReadWriteGrip};
 #[cfg(all(not(target_os = "wasi"), feature = "socketpair"))]
 use {duplex::HalfDuplex, socketpair::AsyncStdSocketpairStream};
+#[cfg(not(windows))]
+use {
+    io_lifetimes::{AsFd, BorrowedFd},
+    unsafe_io::os::posish::{AsRawReadWriteFd, AsReadWriteFd},
+};
 #[cfg(not(target_os = "wasi"))]
 use {
     os_pipe::{PipeReader, PipeWriter},
@@ -173,9 +171,10 @@ impl AsyncStreamReader {
     /// Unlike [`async_std::io::stdin`], this `stdin` returns a stream which is
     /// unbuffered and unlocked.
     ///
-    /// This acquires a [`async_std::io::StdinLock`] (in a non-recursive way) to
-    /// prevent accesses to `async_std::io::Stdin` while this is live, and fails if a
-    /// `AsyncStreamReader` or `AsyncStreamDuplexer` for standard input already exists.
+    /// This acquires a [`async_std::io::StdinLock`] (in a non-recursive way)
+    /// to prevent accesses to `async_std::io::Stdin` while this is live,
+    /// and fails if a `AsyncStreamReader` or `AsyncStreamDuplexer` for
+    /// standard input already exists.
     #[inline]
     pub fn stdin() -> io::Result<Self> {
         todo!("async stdin")
@@ -183,14 +182,15 @@ impl AsyncStreamReader {
 
     /// Read from an open file, taking ownership of it.
     ///
-    /// This method can be passed a [`async_std::fs::File`] or similar `File` types.
+    /// This method can be passed a [`async_std::fs::File`] or similar `File`
+    /// types.
     #[inline]
     #[must_use]
-    pub fn file<Filelike: IntoUnsafeFile + Read + Write + Seek>(filelike: Filelike) -> Self {
+    pub fn file<Filelike: IntoFilelike + Read + Write + Seek>(filelike: Filelike) -> Self {
         // Safety: We don't implement `From`/`Into` to allow the inner `File`
         // to be extracted, so we don't need to worry that we're granting
         // ambient authorities here.
-        Self::_file(File::from_filelike(filelike))
+        Self::_file(File::from_into_filelike(filelike))
     }
 
     #[inline]
@@ -205,8 +205,8 @@ impl AsyncStreamReader {
     /// `TcpStream` types.
     #[inline]
     #[must_use]
-    pub fn tcp_stream<Socketlike: IntoUnsafeSocket>(socketlike: Socketlike) -> Self {
-        Self::_tcp_stream(TcpStream::from_socketlike(socketlike))
+    pub fn tcp_stream<Socketlike: IntoSocketlike>(socketlike: Socketlike) -> Self {
+        Self::_tcp_stream(TcpStream::from_into_socketlike(socketlike))
     }
 
     #[inline]
@@ -287,13 +287,13 @@ impl AsyncStreamReader {
 impl AsyncStreamWriter {
     /// Write to standard output.
     ///
-    /// Unlike [`async_std::io::stdout`], this `stdout` returns a stream which is
-    /// unbuffered and unlocked.
+    /// Unlike [`async_std::io::stdout`], this `stdout` returns a stream which
+    /// is unbuffered and unlocked.
     ///
-    /// This acquires a [`async_std::io::StdoutLock`] (in a non-recursive way) to
-    /// prevent accesses to `async_std::io::Stdout` while this is live, and fails if
-    /// a `AsyncStreamWriter` or `AsyncStreamDuplexer` for standard output already
-    /// exists.
+    /// This acquires a [`async_std::io::StdoutLock`] (in a non-recursive way)
+    /// to prevent accesses to `async_std::io::Stdout` while this is live,
+    /// and fails if a `AsyncStreamWriter` or `AsyncStreamDuplexer` for
+    /// standard output already exists.
     #[inline]
     pub fn stdout() -> io::Result<Self> {
         todo!("async stdout")
@@ -301,14 +301,15 @@ impl AsyncStreamWriter {
 
     /// Write to an open file, taking ownership of it.
     ///
-    /// This method can be passed a [`async_std::fs::File`] or similar `File` types.
+    /// This method can be passed a [`async_std::fs::File`] or similar `File`
+    /// types.
     #[inline]
     #[must_use]
-    pub fn file<Filelike: IntoUnsafeFile + Read + Write + Seek>(filelike: Filelike) -> Self {
+    pub fn file<Filelike: IntoFilelike + Read + Write + Seek>(filelike: Filelike) -> Self {
         // Safety: We don't implement `From`/`Into` to allow the inner `File`
         // to be extracted, so we don't need to worry that we're granting
         // ambient authorities here.
-        Self::_file(File::from_filelike(filelike))
+        Self::_file(File::from_into_filelike(filelike))
     }
 
     #[inline]
@@ -323,11 +324,11 @@ impl AsyncStreamWriter {
     /// `TcpStream` types.
     #[inline]
     #[must_use]
-    pub fn tcp_stream<Socketlike: IntoUnsafeSocket>(socketlike: Socketlike) -> Self {
+    pub fn tcp_stream<Socketlike: IntoSocketlike>(socketlike: Socketlike) -> Self {
         // Safety: We don't implement `From`/`Into` to allow the inner
         // `TcpStream` to be extracted, so we don't need to worry that we're
         // granting ambient authorities here.
-        Self::_tcp_stream(TcpStream::from_socketlike(socketlike))
+        Self::_tcp_stream(TcpStream::from_into_socketlike(socketlike))
     }
 
     #[inline]
@@ -405,14 +406,16 @@ impl AsyncStreamWriter {
 impl AsyncStreamDuplexer {
     /// Duplex with stdin and stdout, taking ownership of them.
     ///
-    /// Unlike [`async_std::io::stdin`] and [`async_std::io::stdout`], this `stdin_stdout`
-    /// returns a stream which is unbuffered and unlocked.
+    /// Unlike [`async_std::io::stdin`] and [`async_std::io::stdout`], this
+    /// `stdin_stdout` returns a stream which is unbuffered and unlocked.
     ///
-    /// This acquires a [`async_std::io::StdinLock`] and a [`async_std::io::StdoutLock`]
-    /// (in non-recursive ways) to prevent accesses to [`async_std::io::Stdin`] and
-    /// [`async_std::io::Stdout`] while this is live, and fails if a `AsyncStreamReader`
-    /// for standard input, a `AsyncStreamWriter` for standard output, or a
-    /// `AsyncStreamDuplexer` for standard input and standard output already exist.
+    /// This acquires a [`async_std::io::StdinLock`] and a
+    /// [`async_std::io::StdoutLock`] (in non-recursive ways) to prevent
+    /// accesses to [`async_std::io::Stdin`] and [`async_std::io::Stdout`]
+    /// while this is live, and fails if a `AsyncStreamReader` for standard
+    /// input, a `AsyncStreamWriter` for standard output, or a
+    /// `AsyncStreamDuplexer` for standard input and standard output already
+    /// exist.
     #[inline]
     pub fn stdin_stdout() -> io::Result<Self> {
         todo!("async stdin_stdout")
@@ -432,8 +435,8 @@ impl AsyncStreamDuplexer {
     /// `TcpStream` types.
     #[inline]
     #[must_use]
-    pub fn tcp_stream<Socketlike: IntoUnsafeSocket>(socketlike: Socketlike) -> Self {
-        Self::_tcp_stream(TcpStream::from_socketlike(socketlike))
+    pub fn tcp_stream<Socketlike: IntoSocketlike>(socketlike: Socketlike) -> Self {
+        Self::_tcp_stream(TcpStream::from_into_socketlike(socketlike))
     }
 
     #[inline]
@@ -763,8 +766,10 @@ impl Read for AsyncStreamDuplexer {
             DuplexResources::SocketpairStream(socketpair_stream) => {
                 Pin::new(socketpair_stream).poll_read(cx, buf)
             }
-            DuplexResources::PipeReaderWriter(_)
-            | DuplexResources::SocketedThreadFunc(_)
+            #[cfg(not(target_os = "wasi"))]
+            DuplexResources::PipeReaderWriter(_) => todo!("async duplex resources"),
+            #[cfg(all(not(target_os = "wasi"), feature = "socketpair"))]
+            DuplexResources::SocketedThreadFunc(_)
             | DuplexResources::SocketedThread(_)
             | DuplexResources::SocketedThreadReadReady(_) => todo!("async duplex resources"),
         }
@@ -1186,13 +1191,9 @@ impl AsRawReadWriteHandleOrSocket for AsyncStreamDuplexer {
     #[inline]
     fn as_raw_write_handle_or_socket(&self) -> RawHandleOrSocket {
         match &self.resources {
-            DuplexResources::TcpStream(tcp_stream) => {
-                Pin::new(tcp_stream).as_raw_handle_or_socket()
-            }
+            DuplexResources::TcpStream(tcp_stream) => tcp_stream.as_raw_handle_or_socket(),
             #[cfg(unix)]
-            DuplexResources::UnixStream(unix_stream) => {
-                Pin::new(unix_stream).as_raw_handle_or_socket()
-            }
+            DuplexResources::UnixStream(unix_stream) => unix_stream.as_raw_handle_or_socket(),
             DuplexResources::StdinStdout(_stdin_stdout) => {
                 todo!("async stdout as_raw_handle_or_socket")
             }
@@ -1203,12 +1204,10 @@ impl AsRawReadWriteHandleOrSocket for AsyncStreamDuplexer {
                 todo!("async child stdout/stdin as_raw_handle_or_socket")
             }
             #[cfg(feature = "char-device")]
-            DuplexResources::CharDevice(char_device) => {
-                Pin::new(char_device).as_raw_handle_or_socket()
-            }
+            DuplexResources::CharDevice(char_device) => char_device.as_raw_handle_or_socket(),
             #[cfg(feature = "socketpair")]
             DuplexResources::SocketpairStream(socketpair_stream) => {
-                Pin::new(socketpair_stream).as_raw_handle_or_socket()
+                socketpair_stream.as_raw_handle_or_socket()
             }
             DuplexResources::PipeReaderWriter(_)
             | DuplexResources::SocketedThreadFunc(_)
@@ -1218,14 +1217,213 @@ impl AsRawReadWriteHandleOrSocket for AsyncStreamDuplexer {
     }
 }
 
-// Safety: AsyncStreamReader owns its handle.
-unsafe impl OwnsRaw for AsyncStreamReader {}
+#[cfg(not(windows))]
+impl AsFd for AsyncStreamReader {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        match &self.resources {
+            ReadResources::File(file) => file.as_fd(),
+            ReadResources::TcpStream(tcp_stream) => tcp_stream.as_fd(),
+            #[cfg(unix)]
+            ReadResources::UnixStream(unix_stream) => unix_stream.as_fd(),
+            ReadResources::PipeReader(_pipe_reader) => todo!("async pipe as_fd"),
+            ReadResources::Stdin(_stdin) => todo!("async stdin as_fd"),
+            #[cfg(not(target_os = "wasi"))]
+            ReadResources::PipedThread(_piped_thread) => todo!("async piped_thread as_fd"),
+            #[cfg(not(target_os = "wasi"))]
+            ReadResources::Child(_child) => todo!("async child as_fd"),
+            #[cfg(not(target_os = "wasi"))]
+            ReadResources::ChildStdout(_child_stdout) => todo!("async child stdout as_fd"),
+            #[cfg(not(target_os = "wasi"))]
+            ReadResources::ChildStderr(_child_stderr) => todo!("async child stderr as_fd"),
+        }
+    }
+}
 
-// Safety: AsyncStreamWriter owns its handle.
-unsafe impl OwnsRaw for AsyncStreamWriter {}
+#[cfg(not(windows))]
+impl AsFd for AsyncStreamWriter {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        match &self.resources {
+            WriteResources::File(file) => file.as_fd(),
+            WriteResources::TcpStream(tcp_stream) => tcp_stream.as_fd(),
+            #[cfg(unix)]
+            WriteResources::UnixStream(unix_stream) => unix_stream.as_fd(),
+            WriteResources::PipeWriter(_pipe_writer) => todo!("async pipe as_fd"),
+            WriteResources::Stdout(_stdout) => todo!("async stdout as_fd"),
+            #[cfg(not(target_os = "wasi"))]
+            WriteResources::PipedThread(_piped_thread) => todo!("async piped_thread as_fd"),
+            #[cfg(not(target_os = "wasi"))]
+            WriteResources::Child(_child) => todo!("async child as_fd"),
+            #[cfg(not(target_os = "wasi"))]
+            WriteResources::ChildStdin(_child_stdin) => todo!("async child stdin as_fd"),
+        }
+    }
+}
 
-// Safety: AsyncStreamDuplexer owns its handle.
-unsafe impl OwnsRaw for AsyncStreamDuplexer {}
+#[cfg(not(windows))]
+impl AsReadWriteFd for AsyncStreamDuplexer {
+    #[inline]
+    fn as_read_fd(&self) -> BorrowedFd<'_> {
+        match &self.resources {
+            DuplexResources::TcpStream(tcp_stream) => tcp_stream.as_fd(),
+            #[cfg(unix)]
+            DuplexResources::UnixStream(unix_stream) => unix_stream.as_fd(),
+            DuplexResources::StdinStdout(_stdin_stdout) => todo!("async stdout as_read_fd"),
+            #[cfg(not(target_os = "wasi"))]
+            DuplexResources::Child(_child) => todo!("async child as_read_fd"),
+            #[cfg(not(target_os = "wasi"))]
+            DuplexResources::ChildStdoutStdin(_stdout_stdin) => {
+                todo!("async child stdout/stdin as_read_fd")
+            }
+            #[cfg(feature = "char-device")]
+            DuplexResources::CharDevice(char_device) => char_device.as_fd(),
+            #[cfg(feature = "socketpair")]
+            DuplexResources::SocketpairStream(socketpair_stream) => socketpair_stream.as_fd(),
+            DuplexResources::PipeReaderWriter(_)
+            | DuplexResources::SocketedThreadFunc(_)
+            | DuplexResources::SocketedThread(_)
+            | DuplexResources::SocketedThreadReadReady(_) => todo!("async duplex resources"),
+        }
+    }
+
+    #[inline]
+    fn as_write_fd(&self) -> BorrowedFd<'_> {
+        match &self.resources {
+            DuplexResources::TcpStream(tcp_stream) => tcp_stream.as_fd(),
+            #[cfg(unix)]
+            DuplexResources::UnixStream(unix_stream) => unix_stream.as_fd(),
+            DuplexResources::StdinStdout(_stdin_stdout) => todo!("async stdout as_write_fd"),
+            #[cfg(not(target_os = "wasi"))]
+            DuplexResources::Child(_child) => todo!("async child as_write_fd"),
+            #[cfg(not(target_os = "wasi"))]
+            DuplexResources::ChildStdoutStdin(_stdout_stdin) => {
+                todo!("async child stdout/stdin as_write_fd")
+            }
+            #[cfg(feature = "char-device")]
+            DuplexResources::CharDevice(char_device) => char_device.as_fd(),
+            #[cfg(feature = "socketpair")]
+            DuplexResources::SocketpairStream(socketpair_stream) => socketpair_stream.as_fd(),
+            DuplexResources::PipeReaderWriter(_)
+            | DuplexResources::SocketedThreadFunc(_)
+            | DuplexResources::SocketedThread(_)
+            | DuplexResources::SocketedThreadReadReady(_) => todo!("async duplex resources"),
+        }
+    }
+}
+
+#[cfg(windows)]
+impl AsHandleOrSocket for AsyncStreamReader {
+    #[inline]
+    fn as_handle_or_socket(&self) -> BorrowedHandleOrSocket<'_> {
+        match &self.resources {
+            ReadResources::File(file) => file.as_handle_or_socket(),
+            ReadResources::TcpStream(tcp_stream) => tcp_stream.as_handle_or_socket(),
+            #[cfg(unix)]
+            ReadResources::UnixStream(unix_stream) => unix_stream.as_handle_or_socket(),
+            ReadResources::PipeReader(_pipe_reader) => todo!("async pipe as_handle_or_socket"),
+            ReadResources::Stdin(_stdin) => todo!("async stdin as_handle_or_socket"),
+            #[cfg(not(target_os = "wasi"))]
+            ReadResources::PipedThread(_piped_thread) => {
+                todo!("async piped_thread as_handle_or_socket")
+            }
+            #[cfg(not(target_os = "wasi"))]
+            ReadResources::Child(_child) => todo!("async child as_handle_or_socket"),
+            #[cfg(not(target_os = "wasi"))]
+            ReadResources::ChildStdout(_child_stdout) => {
+                todo!("async child stdout as_handle_or_socket")
+            }
+            #[cfg(not(target_os = "wasi"))]
+            ReadResources::ChildStderr(_child_stderr) => {
+                todo!("async child stderr as_handle_or_socket")
+            }
+        }
+    }
+}
+
+#[cfg(windows)]
+impl AsHandleOrSocket for AsyncStreamWriter {
+    #[inline]
+    fn as_handle_or_socket(&self) -> BorrowedHandleOrSocket<'_> {
+        match &self.resources {
+            WriteResources::File(file) => file.as_handle_or_socket(),
+            WriteResources::TcpStream(tcp_stream) => tcp_stream.as_handle_or_socket(),
+            #[cfg(unix)]
+            WriteResources::UnixStream(unix_stream) => unix_stream.as_handle_or_socket(),
+            WriteResources::PipeWriter(_pipe_writer) => todo!("async pipe as_handle_or_socket"),
+            WriteResources::Stdout(_stdout) => todo!("async stdout as_handle_or_socket"),
+            #[cfg(not(target_os = "wasi"))]
+            WriteResources::PipedThread(_piped_thread) => {
+                todo!("async piped_thread as_handle_or_socket")
+            }
+            #[cfg(not(target_os = "wasi"))]
+            WriteResources::Child(_child) => todo!("async child as_handle_or_socket"),
+            #[cfg(not(target_os = "wasi"))]
+            WriteResources::ChildStdin(_child_stdin) => {
+                todo!("async child stdin as_handle_or_socket")
+            }
+        }
+    }
+}
+
+#[cfg(windows)]
+impl AsReadWriteHandleOrSocket for AsyncStreamDuplexer {
+    #[inline]
+    fn as_read_handle_or_socket(&self) -> BorrowedHandleOrSocket<'_> {
+        match &self.resources {
+            DuplexResources::TcpStream(tcp_stream) => tcp_stream.as_handle_or_socket(),
+            #[cfg(unix)]
+            DuplexResources::UnixStream(unix_stream) => unix_stream.as_handle_or_socket(),
+            DuplexResources::StdinStdout(_stdin_stdout) => {
+                todo!("async stdout as_handle_or_socket")
+            }
+            #[cfg(not(target_os = "wasi"))]
+            DuplexResources::Child(_child) => todo!("async child as_handle_or_socket"),
+            #[cfg(not(target_os = "wasi"))]
+            DuplexResources::ChildStdoutStdin(_stdout_stdin) => {
+                todo!("async child stdout/stdin as_handle_or_socket")
+            }
+            #[cfg(feature = "char-device")]
+            DuplexResources::CharDevice(char_device) => char_device.as_handle_or_socket(),
+            #[cfg(feature = "socketpair")]
+            DuplexResources::SocketpairStream(socketpair_stream) => {
+                socketpair_stream.as_handle_or_socket()
+            }
+            DuplexResources::PipeReaderWriter(_)
+            | DuplexResources::SocketedThreadFunc(_)
+            | DuplexResources::SocketedThread(_)
+            | DuplexResources::SocketedThreadReadReady(_) => todo!("async duplex resources"),
+        }
+    }
+
+    #[inline]
+    fn as_write_handle_or_socket(&self) -> BorrowedHandleOrSocket<'_> {
+        match &self.resources {
+            DuplexResources::TcpStream(tcp_stream) => tcp_stream.as_handle_or_socket(),
+            #[cfg(unix)]
+            DuplexResources::UnixStream(unix_stream) => unix_stream.as_handle_or_socket(),
+            DuplexResources::StdinStdout(_stdin_stdout) => {
+                todo!("async stdout as_handle_or_socket")
+            }
+            #[cfg(not(target_os = "wasi"))]
+            DuplexResources::Child(_child) => todo!("async child as_handle_or_socket"),
+            #[cfg(not(target_os = "wasi"))]
+            DuplexResources::ChildStdoutStdin(_stdout_stdin) => {
+                todo!("async child stdout/stdin as_handle_or_socket")
+            }
+            #[cfg(feature = "char-device")]
+            DuplexResources::CharDevice(char_device) => char_device.as_handle_or_socket(),
+            #[cfg(feature = "socketpair")]
+            DuplexResources::SocketpairStream(socketpair_stream) => {
+                socketpair_stream.as_handle_or_socket()
+            }
+            DuplexResources::PipeReaderWriter(_)
+            | DuplexResources::SocketedThreadFunc(_)
+            | DuplexResources::SocketedThread(_)
+            | DuplexResources::SocketedThreadReadReady(_) => todo!("async duplex resources"),
+        }
+    }
+}
 
 impl Drop for ReadResources {
     fn drop(&mut self) {
@@ -1284,7 +1482,7 @@ impl Debug for AsyncStreamReader {
         // information about it, because this information is otherwise
         // unavailable to safe Rust code.
         f.debug_struct("AsyncStreamReader")
-            .field("unsafe_handle", &self.as_unsafe_handle())
+            .field("raw_grip", &self.as_raw_grip())
             .finish()
     }
 }
@@ -1295,7 +1493,7 @@ impl Debug for AsyncStreamWriter {
         // information about it, because this information is otherwise
         // unavailable to safe Rust code.
         f.debug_struct("AsyncStreamWriter")
-            .field("unsafe_handle", &self.as_unsafe_handle())
+            .field("raw_grip", &self.as_raw_grip())
             .finish()
     }
 }
@@ -1306,8 +1504,8 @@ impl Debug for AsyncStreamDuplexer {
         // information about it, because this information is otherwise
         // unavailable to safe Rust code.
         f.debug_struct("AsyncStreamDuplexer")
-            .field("unsafe_readable", &self.as_unsafe_read_handle())
-            .field("unsafe_writeable", &self.as_unsafe_write_handle())
+            .field("raw_read_grip", &self.as_raw_read_grip())
+            .field("raw_write_grip", &self.as_raw_write_grip())
             .finish()
     }
 }
