@@ -7,17 +7,20 @@ use duplex::HalfDuplex;
 use layered_io::{default_suggested_buffer_size, Bufferable, HalfDuplexLayered};
 #[cfg(read_initializer)]
 use std::io::Initializer;
-use std::{
-    cmp, fmt,
-    io::{self, BufRead, Error, ErrorKind, IoSlice, IoSliceMut, Read, Write},
-};
+use std::io::{self, BufRead, Error, ErrorKind, IoSlice, IoSliceMut, Read, Write};
+use std::{cmp, fmt};
 #[cfg(not(windows))]
-use unsafe_io::os::posish::{AsRawFd, RawFd};
-use unsafe_io::OwnsRaw;
+use {
+    io_lifetimes::{AsFd, BorrowedFd},
+    unsafe_io::os::posish::{AsRawFd, RawFd},
+};
 #[cfg(windows)]
 use {
+    io_lifetimes::{AsHandle, AsSocket, BorrowedHandle, BorrowedSocket},
     std::os::windows::io::{AsRawHandle, AsRawSocket, RawHandle, RawSocket},
-    unsafe_io::os::windows::{AsRawHandleOrSocket, RawHandleOrSocket},
+    unsafe_io::os::windows::{
+        AsHandleOrSocket, AsRawHandleOrSocket, BorrowedHandleOrSocket, RawHandleOrSocket,
+    },
 };
 
 /// Wraps a reader and writer and buffers their output.
@@ -28,41 +31,43 @@ use {
 /// `BufDuplexer<Inner>` keeps an in-memory buffer of data and writes it to an
 /// underlying writer in large, infrequent batches.
 ///
-/// It can be excessively inefficient to work directly with a [`Read`] instance.
-/// For example, every call to [`read`][`TcpStream::read`] on [`TcpStream`]
-/// results in a system call. A `BufDuplexer<Inner>` performs large, infrequent
-/// reads on the underlying [`Read`] and maintains an in-memory buffer of the results.
+/// It can be excessively inefficient to work directly with a [`Read`]
+/// instance. For example, every call to [`read`][`TcpStream::read`] on
+/// [`TcpStream`] results in a system call. A `BufDuplexer<Inner>` performs
+/// large, infrequent reads on the underlying [`Read`] and maintains an
+/// in-memory buffer of the results.
 ///
-/// `BufDuplexer<Inner>` can improve the speed of programs that make *small* and
-/// *repeated* write calls to the same file or network socket. It does not
+/// `BufDuplexer<Inner>` can improve the speed of programs that make *small*
+/// and *repeated* write calls to the same file or network socket. It does not
 /// help when writing very large amounts at once, or writing just one or a few
 /// times. It also provides no advantage when writing to a destination that is
 /// in memory, like a [`Vec`]`<u8>`.
 ///
-/// `BufDuplexer<Inner>` can improve the speed of programs that make *small* and
-/// *repeated* read calls to the same file or network socket. It does not
+/// `BufDuplexer<Inner>` can improve the speed of programs that make *small*
+/// and *repeated* read calls to the same file or network socket. It does not
 /// help when reading very large amounts at once, or reading just one or a few
 /// times. It also provides no advantage when reading from a source that is
 /// already in memory, like a [`Vec`]`<u8>`.
 ///
-/// It is critical to call [`flush`] before `BufDuplexer<Inner>` is dropped. Though
-/// dropping will attempt to flush the contents of the writer buffer, any errors
-/// that happen in the process of dropping will be ignored. Calling [`flush`]
-/// ensures that the writer buffer is empty and thus dropping will not even attempt
-/// file operations.
+/// It is critical to call [`flush`] before `BufDuplexer<Inner>` is dropped.
+/// Though dropping will attempt to flush the contents of the writer buffer,
+/// any errors that happen in the process of dropping will be ignored. Calling
+/// [`flush`] ensures that the writer buffer is empty and thus dropping will
+/// not even attempt file operations.
 ///
-/// When the `BufDuplexer<Inner>` is dropped, the contents of its reader buffer will be
-/// discarded. Creating multiple instances of a `BufDuplexer<Inner>` on the same
-/// stream can cause data loss. Reading from the underlying reader after
-/// unwrapping the `BufDuplexer<Inner>` with [`BufDuplexer::into_inner`] can also cause
-/// data loss.
+/// When the `BufDuplexer<Inner>` is dropped, the contents of its reader buffer
+/// will be discarded. Creating multiple instances of a `BufDuplexer<Inner>` on
+/// the same stream can cause data loss. Reading from the underlying reader
+/// after unwrapping the `BufDuplexer<Inner>` with [`BufDuplexer::into_inner`]
+/// can also cause data loss.
 ///
 /// # Examples
 ///
 /// Let's write the numbers one through ten to a [`TcpStream`]:
 ///
 /// ```no_run
-/// use std::{io::prelude::*, net::TcpStream};
+/// use std::io::prelude::*;
+/// use std::net::TcpStream;
 ///
 /// let mut stream = TcpStream::connect("127.0.0.1:34254").unwrap();
 ///
@@ -77,7 +82,8 @@ use {
 ///
 /// ```no_run
 /// use io_streams::BufDuplexer;
-/// use std::{io::prelude::*, net::TcpStream};
+/// use std::io::prelude::*;
+/// use std::net::TcpStream;
 ///
 /// let mut stream = BufDuplexer::new(TcpStream::connect("127.0.0.1:34254").unwrap());
 ///
@@ -87,13 +93,14 @@ use {
 /// stream.flush().unwrap();
 /// ```
 ///
-/// By wrapping the stream with a `BufDuplexer<Inner>`, these ten writes are all grouped
-/// together by the buffer and will all be written out in one system call when
-/// the `stream` is flushed.
+/// By wrapping the stream with a `BufDuplexer<Inner>`, these ten writes are
+/// all grouped together by the buffer and will all be written out in one
+/// system call when the `stream` is flushed.
 ///
 /// ```no_run
 /// use io_streams::BufDuplexer;
-/// use std::{io::prelude::*, net::TcpStream};
+/// use std::io::prelude::*;
+/// use std::net::TcpStream;
 ///
 /// fn main() -> std::io::Result<()> {
 ///     let mut stream = BufDuplexer::new(TcpStream::connect("127.0.0.1:34254").unwrap());
@@ -130,8 +137,8 @@ pub(crate) struct BufDuplexerBackend<Inner: HalfDuplex> {
 }
 
 impl<Inner: HalfDuplex> BufDuplexer<Inner> {
-    /// Creates a new `BufDuplexer<Inner>` with default buffer capacities. The default is currently 8 KB,
-    /// but may change in the future.
+    /// Creates a new `BufDuplexer<Inner>` with default buffer capacities. The
+    /// default is currently 8 KB, but may change in the future.
     ///
     /// # Examples
     ///
@@ -148,11 +155,13 @@ impl<Inner: HalfDuplex> BufDuplexer<Inner> {
         }
     }
 
-    /// Creates a new `BufDuplexer<Inner>` with the specified buffer capacities.
+    /// Creates a new `BufDuplexer<Inner>` with the specified buffer
+    /// capacities.
     ///
     /// # Examples
     ///
-    /// Creating a buffer with ten bytes of reader capacity and a writer buffer of a hundered bytes:
+    /// Creating a buffer with ten bytes of reader capacity and a writer buffer
+    /// of a hundered bytes:
     ///
     /// ```no_run
     /// use io_streams::BufDuplexer;
@@ -226,7 +235,8 @@ impl<Inner: HalfDuplex> BufDuplexer<Inner> {
 
     /// Returns a reference to the internally buffered reader data.
     ///
-    /// Unlike [`fill_buf`], this will not attempt to fill the buffer if it is empty.
+    /// Unlike [`fill_buf`], this will not attempt to fill the buffer if it is
+    /// empty.
     ///
     /// [`fill_buf`]: BufRead::fill_buf
     ///
@@ -235,7 +245,8 @@ impl<Inner: HalfDuplex> BufDuplexer<Inner> {
     /// ```no_run
     /// use char_device::CharDevice;
     /// use io_streams::BufDuplexer;
-    /// use std::{fs::File, io::BufRead};
+    /// use std::fs::File;
+    /// use std::io::BufRead;
     ///
     /// fn main() -> std::io::Result<()> {
     ///     let f = CharDevice::new(File::open("/dev/ttyS0")?)?;
@@ -252,7 +263,8 @@ impl<Inner: HalfDuplex> BufDuplexer<Inner> {
         self.inner.reader_buffer()
     }
 
-    /// Returns the number of bytes the internal writer buffer can hold without flushing.
+    /// Returns the number of bytes the internal writer buffer can hold without
+    /// flushing.
     ///
     /// # Examples
     ///
@@ -272,14 +284,16 @@ impl<Inner: HalfDuplex> BufDuplexer<Inner> {
         self.inner.writer_capacity()
     }
 
-    /// Returns the number of bytes the internal reader buffer can hold at once.
+    /// Returns the number of bytes the internal reader buffer can hold at
+    /// once.
     ///
     /// # Examples
     ///
     /// ```no_run
     /// use char_device::CharDevice;
     /// use io_streams::BufDuplexer;
-    /// use std::{fs::File, io::BufRead};
+    /// use std::fs::File;
+    /// use std::io::BufRead;
     ///
     /// fn main() -> std::io::Result<()> {
     ///     let f = CharDevice::new(File::open("/dev/tty")?)?;
@@ -295,13 +309,15 @@ impl<Inner: HalfDuplex> BufDuplexer<Inner> {
         self.inner.reader_capacity()
     }
 
-    /// Unwraps this `BufDuplexer<Inner>`, returning the underlying reader/writer.
+    /// Unwraps this `BufDuplexer<Inner>`, returning the underlying
+    /// reader/writer.
     ///
     /// The buffer is written out before returning the reader/writer.
     ///
     /// # Errors
     ///
-    /// An [`Err`] will be returned if an error occurs while flushing the buffer.
+    /// An [`Err`] will be returned if an error occurs while flushing the
+    /// buffer.
     ///
     /// # Examples
     ///
@@ -751,9 +767,6 @@ impl<Inner: HalfDuplex + AsRawHandleOrSocket> AsRawHandleOrSocket for BufDuplexe
     }
 }
 
-// Safety: `BufDuplexer` implements `OwnsRaw` if `Inner` does.
-unsafe impl<Inner: HalfDuplex + OwnsRaw> OwnsRaw for BufDuplexer<Inner> {}
-
 #[cfg(not(windows))]
 impl<Inner: HalfDuplex + AsRawFd> AsRawFd for BufDuplexerBackend<Inner> {
     #[inline]
@@ -786,8 +799,69 @@ impl<Inner: HalfDuplex + AsRawHandleOrSocket> AsRawHandleOrSocket for BufDuplexe
     }
 }
 
-// Safety: `BufDuplexerBackend` implements `OwnsRaw` if `Inner` does.
-unsafe impl<Inner: HalfDuplex + OwnsRaw> OwnsRaw for BufDuplexerBackend<Inner> {}
+#[cfg(not(windows))]
+impl<Inner: HalfDuplex + AsFd> AsFd for BufDuplexer<Inner> {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.inner.as_fd()
+    }
+}
+
+#[cfg(windows)]
+impl<Inner: HalfDuplex + AsHandle> AsHandle for BufDuplexer<Inner> {
+    #[inline]
+    fn as_handle(&self) -> BorrowedHandle<'_> {
+        self.inner.as_handle()
+    }
+}
+
+#[cfg(windows)]
+impl<Inner: HalfDuplex + AsSocket> AsSocket for BufDuplexer<Inner> {
+    #[inline]
+    fn as_socket(&self) -> BorrowedSocket<'_> {
+        self.inner.as_socket()
+    }
+}
+
+#[cfg(windows)]
+impl<Inner: HalfDuplex + AsHandleOrSocket> AsHandleOrSocket for BufDuplexer<Inner> {
+    #[inline]
+    fn as_handle_or_socket(&self) -> BorrowedHandleOrSocket<'_> {
+        self.inner.as_handle_or_socket()
+    }
+}
+
+#[cfg(not(windows))]
+impl<Inner: HalfDuplex + AsFd> AsFd for BufDuplexerBackend<Inner> {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.inner.as_ref().unwrap().as_fd()
+    }
+}
+
+#[cfg(windows)]
+impl<Inner: HalfDuplex + AsHandle> AsHandle for BufDuplexerBackend<Inner> {
+    #[inline]
+    fn as_handle(&self) -> BorrowedHandle<'_> {
+        self.inner.as_ref().unwrap().as_handle()
+    }
+}
+
+#[cfg(windows)]
+impl<Inner: HalfDuplex + AsSocket> AsSocket for BufDuplexerBackend<Inner> {
+    #[inline]
+    fn as_socket(&self) -> BorrowedSocket<'_> {
+        self.inner.as_ref().unwrap().as_socket()
+    }
+}
+
+#[cfg(windows)]
+impl<Inner: HalfDuplex + AsHandleOrSocket> AsHandleOrSocket for BufDuplexerBackend<Inner> {
+    #[inline]
+    fn as_handle_or_socket(&self) -> BorrowedHandleOrSocket<'_> {
+        self.inner.as_ref().unwrap().as_handle_or_socket()
+    }
+}
 
 #[cfg(feature = "terminal-io")]
 impl<Inner: HalfDuplex + terminal_io::WriteTerminal> terminal_io::Terminal for BufDuplexer<Inner> {}
