@@ -1,39 +1,45 @@
 //! This file is derived from Rust's library/std/src/io/buffered at revision
 //! f7801d6c7cc19ab22bdebcc8efa894a564c53469.
 
-use super::{buf_duplexer::BufDuplexerBackend, BufReaderLineWriterShim, IntoInnerError};
+use super::buf_duplexer::BufDuplexerBackend;
+use super::{BufReaderLineWriterShim, IntoInnerError};
 use duplex::HalfDuplex;
 #[cfg(feature = "layered-io")]
 use layered_io::{Bufferable, HalfDuplexLayered};
+use std::fmt;
 #[cfg(read_initializer)]
 use std::io::Initializer;
-use std::{
-    fmt,
-    io::{self, BufRead, IoSlice, IoSliceMut, Read, Write},
-};
+use std::io::{self, BufRead, IoSlice, IoSliceMut, Read, Write};
 #[cfg(not(windows))]
-use unsafe_io::os::posish::{AsRawFd, RawFd};
-use unsafe_io::OwnsRaw;
+use {
+    io_lifetimes::{AsFd, BorrowedFd},
+    unsafe_io::os::posish::{AsRawFd, RawFd},
+};
 #[cfg(windows)]
 use {
+    io_lifetimes::{AsHandle, AsSocket, BorrowedHandle, BorrowedSocket},
     std::os::windows::io::{AsRawHandle, AsRawSocket, RawHandle, RawSocket},
-    unsafe_io::os::windows::{AsRawHandleOrSocket, RawHandleOrSocket},
+    unsafe_io::os::windows::{
+        AsHandleOrSocket, AsRawHandleOrSocket, BorrowedHandleOrSocket, RawHandleOrSocket,
+    },
 };
 
-/// Wraps a reader and writer and buffers input and output to and from it, flushing
-/// the writer whenever a newline (`0x0a`, `'\n'`) is detected on output.
+/// Wraps a reader and writer and buffers input and output to and from it,
+/// flushing the writer whenever a newline (`0x0a`, `'\n'`) is detected on
+/// output.
 ///
-/// The [`BufDuplexer`] struct wraps a reader and writer and buffers their input and output.
-/// But it only does this batched write when it goes out of scope, or when the
-/// internal buffer is full. Sometimes, you'd prefer to write each line as it's
-/// completed, rather than the entire buffer at once. Enter `BufReaderLineWriter`. It
-/// does exactly that.
+/// The [`BufDuplexer`] struct wraps a reader and writer and buffers their
+/// input and output. But it only does this batched write when it goes out of
+/// scope, or when the internal buffer is full. Sometimes, you'd prefer to
+/// write each line as it's completed, rather than the entire buffer at once.
+/// Enter `BufReaderLineWriter`. It does exactly that.
 ///
-/// Like [`BufDuplexer`], a `BufReaderLineWriter`’s buffer will also be flushed when the
-/// `BufReaderLineWriter` goes out of scope or when its internal buffer is full.
+/// Like [`BufDuplexer`], a `BufReaderLineWriter`’s buffer will also be flushed
+/// when the `BufReaderLineWriter` goes out of scope or when its internal
+/// buffer is full.
 ///
-/// If there's still a partial line in the buffer when the `BufReaderLineWriter` is
-/// dropped, it will flush those contents.
+/// If there's still a partial line in the buffer when the
+/// `BufReaderLineWriter` is dropped, it will flush those contents.
 ///
 /// # Examples
 ///
@@ -43,7 +49,8 @@ use {
 /// ```no_run
 /// use char_device::CharDevice;
 /// use io_streams::BufReaderLineWriter;
-/// use std::{fs, io::prelude::*};
+/// use std::fs;
+/// use std::io::prelude::*;
 ///
 /// fn main() -> std::io::Result<()> {
 ///     let road_not_taken = b"I shall be telling this with a sigh
@@ -120,8 +127,8 @@ impl<Inner: HalfDuplex> BufReaderLineWriter<Inner> {
         }
     }
 
-    /// Creates a new `BufReaderLineWriter` with a specified capacities for the internal
-    /// buffers.
+    /// Creates a new `BufReaderLineWriter` with a specified capacities for the
+    /// internal buffers.
     ///
     /// # Examples
     ///
@@ -198,7 +205,8 @@ impl<Inner: HalfDuplex> BufReaderLineWriter<Inner> {
     ///
     /// # Errors
     ///
-    /// An [`Err`] will be returned if an error occurs while flushing the buffer.
+    /// An [`Err`] will be returned if an error occurs while flushing the
+    /// buffer.
     ///
     /// # Examples
     ///
@@ -495,9 +503,6 @@ impl<Inner: HalfDuplex + AsRawHandleOrSocket> AsRawHandleOrSocket for BufReaderL
     }
 }
 
-// Safety: `BufReaderLineWriter` implements `OwnsRaw` if `Inner` does.
-unsafe impl<Inner: HalfDuplex + OwnsRaw> OwnsRaw for BufReaderLineWriter<Inner> {}
-
 #[cfg(not(windows))]
 impl<Inner: HalfDuplex + AsRawFd> AsRawFd for BufReaderLineWriterBackend<Inner> {
     #[inline]
@@ -532,8 +537,69 @@ impl<Inner: HalfDuplex + AsRawHandleOrSocket> AsRawHandleOrSocket
     }
 }
 
-// Safety: `BufReaderLineWriterBackend` implements `OwnsRaw` if `Inner` does.
-unsafe impl<Inner: HalfDuplex + OwnsRaw> OwnsRaw for BufReaderLineWriterBackend<Inner> {}
+#[cfg(not(windows))]
+impl<Inner: HalfDuplex + AsFd> AsFd for BufReaderLineWriter<Inner> {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.inner.as_fd()
+    }
+}
+
+#[cfg(windows)]
+impl<Inner: HalfDuplex + AsHandle> AsHandle for BufReaderLineWriter<Inner> {
+    #[inline]
+    fn as_handle(&self) -> BorrowedHandle<'_> {
+        self.inner.as_handle()
+    }
+}
+
+#[cfg(windows)]
+impl<Inner: HalfDuplex + AsSocket> AsSocket for BufReaderLineWriter<Inner> {
+    #[inline]
+    fn as_socket(&self) -> BorrowedSocket<'_> {
+        self.inner.as_socket()
+    }
+}
+
+#[cfg(windows)]
+impl<Inner: HalfDuplex + AsHandleOrSocket> AsHandleOrSocket for BufReaderLineWriter<Inner> {
+    #[inline]
+    fn as_handle_or_socket(&self) -> BorrowedHandleOrSocket<'_> {
+        self.inner.as_handle_or_socket()
+    }
+}
+
+#[cfg(not(windows))]
+impl<Inner: HalfDuplex + AsFd> AsFd for BufReaderLineWriterBackend<Inner> {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.inner.as_fd()
+    }
+}
+
+#[cfg(windows)]
+impl<Inner: HalfDuplex + AsHandle> AsHandle for BufReaderLineWriterBackend<Inner> {
+    #[inline]
+    fn as_handle(&self) -> BorrowedHandle<'_> {
+        self.inner.as_handle()
+    }
+}
+
+#[cfg(windows)]
+impl<Inner: HalfDuplex + AsSocket> AsSocket for BufReaderLineWriterBackend<Inner> {
+    #[inline]
+    fn as_socket(&self) -> BorrowedSocket<'_> {
+        self.inner.as_socket()
+    }
+}
+
+#[cfg(windows)]
+impl<Inner: HalfDuplex + AsHandleOrSocket> AsHandleOrSocket for BufReaderLineWriterBackend<Inner> {
+    #[inline]
+    fn as_handle_or_socket(&self) -> BorrowedHandleOrSocket<'_> {
+        self.inner.as_handle_or_socket()
+    }
+}
 
 #[cfg(feature = "terminal-io")]
 impl<Inner: HalfDuplex + terminal_io::Terminal> terminal_io::Terminal
